@@ -1,6 +1,10 @@
 package compile
 
 import (
+	"io/fs"
+	"os"
+	"strings"
+
 	"github.com/open-policy-agent/opa/v1/ast"
 
 	"github.com/styrainc/regal/internal/embeds"
@@ -30,18 +34,32 @@ func Capabilities() *ast.Capabilities {
 	return caps
 }
 
+// RegalSchemaSet returns a SchemaSet containing the Regal schemas embedded in the binary.
+// Currently only used by the test command. Should we want to expand the use of this later,
+// we'll probably want to only read the schemas relevant to the context.
 func RegalSchemaSet() *ast.SchemaSet {
 	schemaSet := ast.NewSchemaSet()
-	astSchema := util.Must(embeds.SchemasFS.ReadFile("schemas/regal-ast.json"))
-	aggSchema := util.Must(embeds.SchemasFS.ReadFile("schemas/aggregate.json"))
 
-	var astSchemaAny, aggSchemaAny any
+	_ = fs.WalkDir(embeds.SchemasFS, "schemas", func(path string, entry fs.DirEntry, err error) error {
+		if err != nil {
+			return err
+		}
 
-	util.Must0(encoding.JSON().Unmarshal(astSchema, &astSchemaAny))
-	schemaSet.Put(ast.MustParseRef("schema.regal.ast"), astSchemaAny)
+		if !strings.HasSuffix(entry.Name(), ".json") {
+			return nil
+		}
 
-	util.Must0(encoding.JSON().Unmarshal(aggSchema, &aggSchemaAny))
-	schemaSet.Put(ast.MustParseRef("schema.regal.aggregate"), aggSchemaAny)
+		var schemaAny any
+
+		util.Must0(encoding.JSON().Unmarshal(util.Must(embeds.SchemasFS.ReadFile(path)), &schemaAny))
+
+		spl := strings.Split(strings.TrimSuffix(path, ".json"), string(os.PathSeparator))
+		ref := ast.Ref([]*ast.Term{ast.SchemaRootDocument}).Extend(ast.MustParseRef(strings.Join(spl[1:], ".")))
+
+		schemaSet.Put(ref, schemaAny)
+
+		return nil
+	})
 
 	return schemaSet
 }
