@@ -4,28 +4,9 @@ import (
 	"slices"
 	"testing"
 
-	"github.com/open-policy-agent/opa/util/test"
+	"github.com/open-policy-agent/opa/v1/util/test"
 )
 
-func TestJSONRoundTrip(t *testing.T) {
-	t.Parallel()
-
-	type foo struct {
-		Bar string `json:"bar"`
-	}
-
-	m := map[string]any{"bar": "foo"}
-	f := foo{}
-
-	if err := JSONRoundTrip(m, &f); err != nil {
-		t.Fatal(err)
-	}
-
-	if f.Bar != "foo" {
-		t.Errorf("expected JSON roundtrip to set struct value")
-	}
-}
-
 func TestFindManifestLocations(t *testing.T) {
 	t.Parallel()
 
