@@ -8,7 +8,7 @@ import (
 
 // BuiltinsForCapabilities returns a list of builtins from the provided capabilities.
 func BuiltinsForCapabilities(capabilities *ast.Capabilities) map[string]*ast.Builtin {
-	m := make(map[string]*ast.Builtin)
+	m := make(map[string]*ast.Builtin, len(capabilities.Builtins))
 	for _, b := range capabilities.Builtins {
 		m[b.Name] = b
 	}
@@ -18,10 +18,9 @@ func BuiltinsForCapabilities(capabilities *ast.Capabilities) map[string]*ast.Bui
 
 func BuiltinCategory(builtin *ast.Builtin) (category string) {
 	if len(builtin.Categories) == 0 {
-		if s := strings.Split(builtin.Name, "."); len(s) > 1 {
-			category = s[0]
-		} else {
-			category = builtin.Name
+		category = builtin.Name
+		if i := strings.Index(builtin.Name, "."); i > -1 {
+			category = builtin.Name[:i]
 		}
 	} else {
 		category = builtin.Categories[0]
