@@ -4,7 +4,6 @@ import (
 	"context"
 	"errors"
 	"fmt"
-	"strings"
 	"sync"
 
 	"github.com/open-policy-agent/opa/v1/ast"
@@ -13,14 +12,30 @@ import (
 	rbundle "github.com/styrainc/regal/bundle"
 	"github.com/styrainc/regal/internal/lsp/clients"
 	"github.com/styrainc/regal/internal/lsp/types"
-	"github.com/styrainc/regal/internal/lsp/uri"
-	"github.com/styrainc/regal/internal/parse"
 	"github.com/styrainc/regal/pkg/builtins"
 
 	"github.com/styrainc/roast/pkg/encoding"
+	"github.com/styrainc/roast/pkg/rast"
 	"github.com/styrainc/roast/pkg/transform"
 )
 
+var (
+	emptyResult = rego.Result{}
+
+	errNoResults          = errors.New("no results returned from evaluation")
+	errExcpectedOneResult = errors.New("expected exactly one result from evaluation")
+	errExcpectedOneExpr   = errors.New("expected exactly one expression in result")
+)
+
+func init() {
+	ast.InternStringTerm(
+		// All keys from Code Actions
+		"identifier", "workspace_root_uri", "web_server_base_uri", "client", "params", "start", "end",
+		"textDocument", "context", "range", "uri", "diagnostics", "only", "triggerKind", "codeDescription",
+		"message", "severity", "source", "code", "data", "title", "command", "kind", "isPreferred",
+	)
+}
+
 type BuiltInCall struct {
 	Builtin  *ast.Builtin
 	Location *ast.Location
@@ -39,6 +54,33 @@ type KeywordUseLocation struct {
 	Col uint `json:"col"`
 }
 
+type Client struct {
+	Identifier            clients.Identifier           `json:"identifier"`
+	InitializationOptions *types.InitializationOptions `json:"init_options,omitempty"`
+}
+
+type Environment struct {
+	WorkspaceRootURI string `json:"workspace_root_uri"`
+	WebServerBaseURI string `json:"web_server_base_uri"`
+}
+
+type RegalContext struct {
+	Client           Client `json:"client"`
+	WebServerBaseURI string `json:"web_server_base_uri"`
+	WorkspaceRootURI string `json:"workspace_root_uri"`
+}
+
+type CodeActionInput struct {
+	Regal  RegalContext           `json:"regal"`
+	Params types.CodeActionParams `json:"params"`
+}
+
+type CodeActionContext struct {
+	Client           clients.Identifier `json:"client"`
+	WebServerBaseURI string             `json:"web_server_base_uri"`
+	WorkspaceRootURI string             `json:"workspace_root_uri"`
+}
+
 func PositionFromLocation(loc *ast.Location) types.Position {
 	//nolint:gosec
 	return types.Position{
@@ -102,6 +144,7 @@ var (
 	keywordsPreparedQuery          *rego.PreparedEvalQuery
 	ruleHeadLocationsPreparedQuery *rego.PreparedEvalQuery
 	codeLensPreparedQuery          *rego.PreparedEvalQuery
+	codeActionPreparedQuery        *rego.PreparedEvalQuery
 )
 
 var preparedQueriesInitOnce sync.Once
@@ -113,41 +156,10 @@ type policy struct {
 }
 
 func initialize() {
-	createArgs := func(args ...func(*rego.Rego)) []func(*rego.Rego) {
-		always := append([]func(*rego.Rego){
-			rego.ParsedBundle("regal", &rbundle.LoadedBundle),
-			rego.StoreReadAST(true),
-		}, builtins.RegalBuiltinRegoFuncs...)
-
-		return append(always, args...)
-	}
-
-	keywordRegoArgs := createArgs(rego.Query("data.regal.ast.keywords"))
-
-	kwpq, err := rego.New(keywordRegoArgs...).PrepareForEval(context.Background())
-	if err != nil {
-		panic(err)
-	}
-
-	keywordsPreparedQuery = &kwpq
-
-	ruleHeadLocationsRegoArgs := createArgs(rego.Query("data.regal.ast.rule_head_locations"))
-
-	rhlpq, err := rego.New(ruleHeadLocationsRegoArgs...).PrepareForEval(context.Background())
-	if err != nil {
-		panic(err)
-	}
-
-	ruleHeadLocationsPreparedQuery = &rhlpq
-
-	codeLensRegoArgs := createArgs(rego.Query("data.regal.lsp.codelens.lenses"))
-
-	clpq, err := rego.New(codeLensRegoArgs...).PrepareForEval(context.Background())
-	if err != nil {
-		panic(err)
-	}
-
-	codeLensPreparedQuery = &clpq
+	keywordsPreparedQuery = createPreparedQuery("data.regal.ast.keywords")
+	codeLensPreparedQuery = createPreparedQuery("data.regal.lsp.codelens.lenses")
+	codeActionPreparedQuery = createPreparedQuery("data.regal.lsp.codeaction.actions")
+	ruleHeadLocationsPreparedQuery = createPreparedQuery("data.regal.ast.rule_head_locations")
 }
 
 // AllKeywords returns all keywords in the module.
@@ -192,18 +204,38 @@ func CodeLenses(ctx context.Context, uri, contents string, module *ast.Module) (
 	return value, nil
 }
 
+// CodeActions returns all code actions in the module.
+// Note that at least as of now, no code actions depend on the data in the module, so
+// it is not passed as part of the input. This could change in the future.
+func CodeActions(ctx context.Context, input CodeActionInput) ([]types.CodeAction, error) {
+	preparedQueriesInitOnce.Do(initialize)
+
+	var codeActions []types.CodeAction
+
+	value, err := queryToValueWithParsedInput(ctx, codeActionPreparedQuery, rast.StructToValue(input), codeActions)
+	if err != nil {
+		return nil, fmt.Errorf("failed querying code lenses: %w", err)
+	}
+
+	return value, nil
+}
+
 func queryToValue[T any](ctx context.Context, pq *rego.PreparedEvalQuery, policy policy, toValue T) (T, error) {
-	input, err := parse.PrepareAST(policy.fileName, policy.contents, policy.module)
+	input, err := transform.ToAST(policy.fileName, policy.contents, policy.module, false)
 	if err != nil {
 		return toValue, fmt.Errorf("failed to prepare input: %w", err)
 	}
 
-	inputValue, err := transform.ToOPAInputValue(input)
-	if err != nil {
-		return toValue, fmt.Errorf("failed converting input to value: %w", err)
-	}
+	return queryToValueWithParsedInput(ctx, pq, input, toValue)
+}
 
-	result, err := toValidResult(pq.Eval(ctx, rego.EvalParsedInput(inputValue)))
+func queryToValueWithParsedInput[T any](
+	ctx context.Context,
+	pq *rego.PreparedEvalQuery,
+	input ast.Value,
+	toValue T,
+) (T, error) {
+	result, err := toValidResult(pq.Eval(ctx, rego.EvalParsedInput(input)))
 	if err != nil {
 		return toValue, err
 	}
@@ -216,78 +248,49 @@ func queryToValue[T any](ctx context.Context, pq *rego.PreparedEvalQuery, policy
 }
 
 func toValidResult(rs rego.ResultSet, err error) (rego.Result, error) {
-	if err != nil {
-		return rego.Result{}, fmt.Errorf("evaluation failed: %w", err)
-	}
-
-	if len(rs) == 0 {
-		return rego.Result{}, errors.New("no results returned from evaluation")
-	}
-
-	if len(rs) != 1 {
-		return rego.Result{}, errors.New("expected exactly one result from evaluation")
-	}
-
-	if len(rs[0].Expressions) != 1 {
-		return rego.Result{}, errors.New("expected exactly one expression in result")
+	switch {
+	case err != nil:
+		return emptyResult, fmt.Errorf("evaluation failed: %w", err)
+	case len(rs) == 0:
+		return emptyResult, errNoResults
+	case len(rs) != 1:
+		return emptyResult, errExcpectedOneResult
+	case len(rs[0].Expressions) != 1:
+		return emptyResult, errExcpectedOneExpr
 	}
 
 	return rs[0], nil
 }
 
-// ToInput prepares a module with Regal additions to be used as input for evaluation.
-func ToInput(
-	fileURI string,
-	cid clients.Identifier,
-	content string,
-	context map[string]any,
-) (map[string]any, error) {
-	path := uri.ToPath(cid, fileURI)
-
-	input := map[string]any{
-		"regal": map[string]any{
-			"file": map[string]any{
-				"name":  path,
-				"uri":   fileURI,
-				"lines": strings.Split(content, "\n"),
-			},
-			"context": context,
-		},
-	}
-
-	if regal, ok := input["regal"].(map[string]any); ok {
-		if f, ok := regal["file"].(map[string]any); ok {
-			f["uri"] = fileURI
-		}
-
-		regal["client_id"] = cid
-	}
-
-	return SetInputContext(input, context), nil
-}
-
-func SetInputContext(input map[string]any, context map[string]any) map[string]any {
-	if regal, ok := input["regal"].(map[string]any); ok {
-		regal["context"] = context
-	}
-
-	return input
-}
-
-func QueryRegalBundle(ctx context.Context, input map[string]any, pq rego.PreparedEvalQuery) (map[string]any, error) {
-	inputValue, err := transform.ToOPAInputValue(input)
-	if err != nil {
-		return nil, fmt.Errorf("failed converting input map to value: %w", err)
-	}
-
-	result, err := pq.Eval(ctx, rego.EvalParsedInput(inputValue))
+func QueryRegalBundle(ctx context.Context, input ast.Value, pq rego.PreparedEvalQuery) (map[string]any, error) {
+	result, err := pq.Eval(ctx, rego.EvalParsedInput(input))
 	if err != nil {
 		return nil, fmt.Errorf("failed evaluating query: %w", err)
 	}
 
 	if len(result) == 0 {
-		return nil, errors.New("expected result from evaluation, didn't get it")
+		return nil, errNoResults
 	}
 
 	return result[0].Bindings, nil
 }
+
+func createArgs(args ...func(*rego.Rego)) []func(*rego.Rego) {
+	always := append([]func(*rego.Rego){
+		rego.ParsedBundle("regal", &rbundle.LoadedBundle),
+		rego.StoreReadAST(true),
+	}, builtins.RegalBuiltinRegoFuncs...)
+
+	return append(always, args...)
+}
+
+func createPreparedQuery(query string) *rego.PreparedEvalQuery {
+	args := createArgs(rego.ParsedQuery(rast.RefStringToBody(query)))
+
+	pq, err := rego.New(args...).PrepareForEval(context.Background())
+	if err != nil {
+		panic(fmt.Sprintf("failed to prepare query %s: %v", query, err))
+	}
+
+	return &pq
+}
