@@ -197,7 +197,7 @@ func (l *LanguageServer) Handle(ctx context.Context, _ *jsonrpc2.Conn, req *json
 	case "initialized":
 		return l.handleInitialized()
 	case "textDocument/codeAction":
-		return handler.WithParams(req, l.handleTextDocumentCodeAction)
+		return handler.WithContextAndParams(ctx, req, l.handleTextDocumentCodeAction)
 	case "textDocument/definition":
 		return handler.WithParams(req, l.handleTextDocumentDefinition)
 	case "textDocument/diagnostic":
@@ -930,7 +930,11 @@ func (l *LanguageServer) StartWorkspaceStateWorker(ctx context.Context) {
 			// next, check if there are any new files that are not ignored and
 			// need to be loaded. We get new only so that files being worked
 			// on are not loaded from disk during editing.
-			newURIs, err := l.loadWorkspaceContents(ctx, true)
+			newURIs, failed, err := l.loadWorkspaceContents(ctx, true)
+			for _, f := range failed {
+				l.logf(log.LevelMessage, "failed to load file %s: %s", f.URI, f.Error)
+			}
+
 			if err != nil {
 				l.logf(log.LevelMessage, "failed to refresh workspace contents: %s", err)
 
@@ -1429,8 +1433,13 @@ func (l *LanguageServer) handleTextDocumentHover(params types.TextDocumentHoverP
 				// this is an approximation, if there are multiple violations on the same line
 				// where hover loc is in their range, then they all just share a range as a
 				// single range is needed in the hover response.
+				source := ""
+				if v.Source != nil {
+					source = *v.Source
+				}
+
 				sharedRange = v.Range
-				docSnippets = append(docSnippets, fmt.Sprintf("[%s/%s](%s)", v.Source, v.Code, v.CodeDescription.Href))
+				docSnippets = append(docSnippets, fmt.Sprintf("[%s/%s](%s)", source, v.Code, v.CodeDescription.Href))
 			}
 		}
 
@@ -1516,104 +1525,22 @@ func (l *LanguageServer) handleTextDocumentHover(params types.TextDocumentHoverP
 	return nil, nil
 }
 
-func (l *LanguageServer) handleTextDocumentCodeAction(params types.CodeActionParams) (any, error) {
+func (l *LanguageServer) handleTextDocumentCodeAction(ctx context.Context, params types.CodeActionParams) (any, error) {
 	if l.ignoreURI(params.TextDocument.URI) {
 		return noCodeActions, nil
 	}
 
-	actions := []types.CodeAction{}
-
-	// only VS Code has the capability to open a provided URL, as far as we know
-	// if we learn about others with this capability later, we should add them!
-	if l.clientIdentifier == clients.IdentifierVSCode {
-		explorerURL := l.webServer.GetBaseURL() + "/explorer" +
-			strings.TrimPrefix(params.TextDocument.URI, l.workspaceRootURI)
-
-		actions = append(actions, types.CodeAction{
-			Title: "Explore compiler stages for this policy",
-			Kind:  "source.explore",
-			Command: types.Command{
-				Title:     "Explore compiler stages for this policy",
-				Command:   "vscode.open",
-				Arguments: &[]any{explorerURL},
+	return rego.CodeActions(ctx, rego.CodeActionInput{ //nolint:wrapcheck
+		Regal: rego.RegalContext{
+			Client: rego.Client{
+				Identifier:            l.clientIdentifier,
+				InitializationOptions: &l.clientInitializationOptions,
 			},
-		})
-	}
-
-	for _, diag := range params.Context.Diagnostics {
-		switch diag.Code {
-		case ruleNameOPAFmt:
-			actions = append(actions, types.CodeAction{
-				Title:       "Format using opa fmt",
-				Kind:        "quickfix",
-				Diagnostics: []types.Diagnostic{diag},
-				IsPreferred: truePtr,
-				Command:     FmtCommand(params.TextDocument.URI),
-			})
-		case ruleNameUseRegoV1:
-			actions = append(actions, types.CodeAction{
-				Title:       "Format for Rego v1 using opa fmt",
-				Kind:        "quickfix",
-				Diagnostics: []types.Diagnostic{diag},
-				IsPreferred: truePtr,
-				Command:     FmtV1Command(params.TextDocument.URI),
-			})
-		case ruleNameUseAssignmentOperator:
-			actions = append(actions, types.CodeAction{
-				Title:       "Replace = with := in assignment",
-				Kind:        "quickfix",
-				Diagnostics: []types.Diagnostic{diag},
-				IsPreferred: truePtr,
-				Command:     UseAssignmentOperatorCommand(params.TextDocument.URI, diag),
-			})
-		case ruleNameNoWhitespaceComment:
-			actions = append(actions, types.CodeAction{
-				Title:       "Format comment to have leading whitespace",
-				Kind:        "quickfix",
-				Diagnostics: []types.Diagnostic{diag},
-				IsPreferred: truePtr,
-				Command:     NoWhiteSpaceCommentCommand(params.TextDocument.URI, diag),
-			})
-		case ruleNameDirectoryPackageMismatch:
-			actions = append(actions, types.CodeAction{
-				Title:       "Move file so that directory structure mirrors package path",
-				Kind:        "quickfix",
-				Diagnostics: []types.Diagnostic{diag},
-				IsPreferred: truePtr,
-				Command:     DirectoryStructureMismatchCommand(params.TextDocument.URI, diag),
-			})
-		case ruleNameNonRawRegexPattern:
-			actions = append(actions, types.CodeAction{
-				Title:       "Replace \" with ` in regex pattern",
-				Kind:        "quickfix",
-				Diagnostics: []types.Diagnostic{diag},
-				IsPreferred: truePtr,
-				Command:     NonRawRegexPatternCommand(params.TextDocument.URI, diag),
-			})
-		}
-
-		if diag.CodeDescription == nil {
-			l.logf(log.LevelMessage, "missing code description and href for %q", diag.Code)
-		}
-
-		if l.clientIdentifier == clients.IdentifierVSCode && diag.CodeDescription != nil {
-			// always show the docs link
-			txt := "Show documentation for " + diag.Code
-			actions = append(actions, types.CodeAction{
-				Title:       txt,
-				Kind:        "quickfix",
-				Diagnostics: []types.Diagnostic{diag},
-				IsPreferred: truePtr,
-				Command: types.Command{
-					Title:     txt,
-					Command:   "vscode.open",
-					Arguments: &[]any{diag.CodeDescription.Href},
-				},
-			})
-		}
-	}
-
-	return actions, nil
+			WebServerBaseURI: l.webServer.GetBaseURL(),
+			WorkspaceRootURI: l.workspaceRootURI,
+		},
+		Params: params,
+	})
 }
 
 func (l *LanguageServer) handleWorkspaceExecuteCommand(params types.ExecuteCommandParams) (any, error) {
@@ -1647,8 +1574,6 @@ func (l *LanguageServer) handleTextDocumentInlayHint(params types.TextDocumentIn
 		return partialInlayHints(parseErrors, contents, params.TextDocument.URI, bis), nil
 	}
 
-	// TODO: use GetContentAndModule here, or do we need to handle the cases separately?
-	// file is blank, nothing to do
 	if contents, ok := l.cache.GetFileContents(params.TextDocument.URI); ok && contents == "" {
 		return []types.InlayHint{}, nil
 	}
@@ -1660,9 +1585,7 @@ func (l *LanguageServer) handleTextDocumentInlayHint(params types.TextDocumentIn
 		return []types.InlayHint{}, nil
 	}
 
-	inlayHints := getInlayHints(module, bis)
-
-	return inlayHints, nil
+	return getInlayHints(module, bis), nil
 }
 
 func (l *LanguageServer) handleTextDocumentCodeLens(ctx context.Context, params types.CodeLensParams) (any, error) {
@@ -1699,12 +1622,11 @@ func (l *LanguageServer) handleTextDocumentCodeLens(ctx context.Context, params
 		return nil, fmt.Errorf("failed to get code lenses: %w", err)
 	}
 
-	if l.clientInitializationOptions.EnableDebugCodelens != nil &&
-		*l.clientInitializationOptions.EnableDebugCodelens {
+	if l.clientInitializationOptions.EnableDebugCodelens != nil && *l.clientInitializationOptions.EnableDebugCodelens {
 		return lenses, nil
 	}
 
-	// filter out `regal.debug` codelens
+	// remove `regal.debug` codelens, as it's not enabled here
 	filteredLenses := make([]types.CodeLens, 0, len(lenses))
 
 	for _, lens := range lenses {
@@ -2194,10 +2116,7 @@ func (l *LanguageServer) handleWorkspaceDidRenameFiles(
 
 		l.cache.SetFileContents(renameOp.NewURI, content)
 
-		job := lintFileJob{
-			Reason: "textDocument/didRename",
-			URI:    renameOp.NewURI,
-		}
+		job := lintFileJob{Reason: "textDocument/didRename", URI: renameOp.NewURI}
 
 		l.lintFileJobs <- job
 		l.builtinsPositionJobs <- job
@@ -2291,21 +2210,15 @@ func (l *LanguageServer) handleInitialize(ctx context.Context, params types.Init
 		l.clientInitializationOptions = *params.InitializationOptions
 	}
 
-	regoFilter := types.FileOperationFilter{
-		Scheme: "file",
-		Pattern: types.FileOperationPattern{
-			Glob: "**/*.rego",
-		},
-	}
+	regoFilter := types.FileOperationFilter{Scheme: "file", Pattern: types.FileOperationPattern{Glob: "**/*.rego"}}
+	fileOpOpts := types.FileOperationRegistrationOptions{Filters: []types.FileOperationFilter{regoFilter}}
 
 	initializeResult := types.InitializeResult{
 		Capabilities: types.ServerCapabilities{
 			TextDocumentSyncOptions: types.TextDocumentSyncOptions{
 				OpenClose: true,
 				Change:    1, // TODO: write logic to use 2, for incremental updates
-				Save: types.TextDocumentSaveOptions{
-					IncludeText: true,
-				},
+				Save:      types.TextDocumentSaveOptions{IncludeText: true},
 			},
 			DiagnosticProvider: types.DiagnosticOptions{
 				Identifier:            "rego",
@@ -2314,15 +2227,9 @@ func (l *LanguageServer) handleInitialize(ctx context.Context, params types.Init
 			},
 			Workspace: types.WorkspaceOptions{
 				FileOperations: types.FileOperationsServerCapabilities{
-					DidCreate: types.FileOperationRegistrationOptions{
-						Filters: []types.FileOperationFilter{regoFilter},
-					},
-					DidRename: types.FileOperationRegistrationOptions{
-						Filters: []types.FileOperationFilter{regoFilter},
-					},
-					DidDelete: types.FileOperationRegistrationOptions{
-						Filters: []types.FileOperationFilter{regoFilter},
-					},
+					DidCreate: fileOpOpts,
+					DidRename: fileOpOpts,
+					DidDelete: fileOpOpts,
 				},
 				WorkspaceFolders: types.WorkspaceFoldersServerCapabilities{
 					// NOTE(anders): The language server protocol doesn't go into detail about what this is meant to
@@ -2337,16 +2244,9 @@ func (l *LanguageServer) handleInitialize(ctx context.Context, params types.Init
 					Supported: true,
 				},
 			},
-			InlayHintProvider: types.InlayHintOptions{
-				ResolveProvider: false,
-			},
-			HoverProvider: true,
-			CodeActionProvider: types.CodeActionOptions{
-				CodeActionKinds: []string{
-					"quickfix",
-					"source.explore",
-				},
-			},
+			InlayHintProvider:  types.InlayHintOptions{ResolveProvider: false},
+			HoverProvider:      true,
+			CodeActionProvider: types.CodeActionOptions{CodeActionKinds: []string{"quickfix", "source.explore"}},
 			ExecuteCommandProvider: types.ExecuteCommandOptions{
 				Commands: []string{
 					"regal.debug",
@@ -2416,8 +2316,13 @@ func (l *LanguageServer) handleInitialize(ctx context.Context, params types.Init
 			l.logf(log.LevelMessage, "no config file found for workspace: %s", err)
 		}
 
-		if _, err = l.loadWorkspaceContents(ctx, false); err != nil {
-			return nil, fmt.Errorf("failed to load workspace contents: %w", err)
+		_, failed, err := l.loadWorkspaceContents(ctx, false)
+		for _, f := range failed {
+			l.logf(log.LevelMessage, "failed to load file %s: %s", f.URI, f.Error)
+		}
+
+		if err != nil {
+			l.logf(log.LevelMessage, "failed to load workspace contents: %s", err)
 		}
 
 		l.webServer.SetWorkspaceURI(l.workspaceRootURI)
@@ -2434,10 +2339,18 @@ func (l *LanguageServer) handleInitialize(ctx context.Context, params types.Init
 	return initializeResult, nil
 }
 
-func (l *LanguageServer) loadWorkspaceContents(ctx context.Context, newOnly bool) ([]string, error) {
+type loadWorkspaceContentsFailedFile struct {
+	URI   string
+	Error error
+}
+
+func (l *LanguageServer) loadWorkspaceContents(ctx context.Context, newOnly bool) (
+	[]string, []loadWorkspaceContentsFailedFile, error,
+) {
 	workspaceRootPath := l.workspacePath()
 
 	changedOrNewURIs := make([]string, 0)
+	failed := make([]loadWorkspaceContentsFailedFile, 0)
 
 	if err := rio.WalkFiles(workspaceRootPath, func(path string) error {
 		fileURI := uri.FromPath(l.clientIdentifier, path)
@@ -2464,7 +2377,13 @@ func (l *LanguageServer) loadWorkspaceContents(ctx context.Context, newOnly bool
 
 		changed, _, err := cache.UpdateCacheForURIFromDisk(l.cache, fileURI, path)
 		if err != nil {
-			return fmt.Errorf("failed to update cache for uri %q: %w", path, err)
+			failed = append(failed, loadWorkspaceContentsFailedFile{
+				URI:   fileURI,
+				Error: fmt.Errorf("failed to update cache for uri %q: %w", path, err),
+			})
+
+			// continue processing other files
+			return nil
 		}
 
 		// there is no need to update the parse if the file contents
@@ -2476,23 +2395,29 @@ func (l *LanguageServer) loadWorkspaceContents(ctx context.Context, newOnly bool
 		bis := l.builtinsForCurrentCapabilities()
 
 		if _, err = updateParse(ctx, l.cache, l.regoStore, fileURI, bis, l.regoVersionForURI(fileURI)); err != nil {
-			return fmt.Errorf("failed to update parse: %w", err)
+			failed = append(failed, loadWorkspaceContentsFailedFile{
+				URI:   fileURI,
+				Error: fmt.Errorf("failed to update parse: %w", err),
+			})
+
+			// continue processing other files
+			return nil
 		}
 
 		changedOrNewURIs = append(changedOrNewURIs, fileURI)
 
 		return nil
 	}); err != nil {
-		return nil, fmt.Errorf("failed to walk workspace dir %q: %w", workspaceRootPath, err)
+		return nil, nil, fmt.Errorf("failed to walk workspace dir %q: %w", workspaceRootPath, err)
 	}
 
 	if l.bundleCache != nil {
 		if _, err := l.bundleCache.Refresh(); err != nil {
-			return nil, fmt.Errorf("failed to refresh the bundle cache: %w", err)
+			return nil, nil, fmt.Errorf("failed to refresh the bundle cache: %w", err)
 		}
 	}
 
-	return changedOrNewURIs, nil
+	return changedOrNewURIs, failed, nil
 }
 
 func (l *LanguageServer) handleInitialized() (any, error) {
