@@ -1,8 +1,7 @@
-//nolint:gochecknoglobals
 package test
 
 import (
-	"github.com/open-policy-agent/opa/ast"
+	"github.com/open-policy-agent/opa/v1/ast"
 
 	"github.com/styrainc/regal/internal/parse"
 	"github.com/styrainc/regal/pkg/rules"
