@@ -53,6 +53,12 @@ func (s *Server) GetBaseURL() string {
 	return s.baseURL
 }
 
+// SetBaseURL sets the base URL for the server
+// NOTE: This is normally set by the server itself, and this method is provided only for testing purposes.
+func (s *Server) SetBaseURL(baseURL string) {
+	s.baseURL = baseURL
+}
+
 type state struct {
 	Code   string
 	Result []stringResult
